<?php

namespace App\Http\Controllers\Api\Exam;

use App\Http\Controllers\Controller;
use App\Repositories\Criteria\Exam\TopicCriteria;
use App\Repositories\Enums\ResponseCodeEnum;
use App\Repositories\Models\Exam\Topic;
use App\Repositories\Presenters\Exam\TopicPresenter;
use Illuminate\Http\Request;
use Jiannei\Response\Laravel\Support\Facades\Response;
use Prettus\Validator\Contracts\ValidatorInterface;
use App\Contracts\Repositories\Exam\TopicRepository;
use App\Repositories\Validators\Exam\TopicValidator;

/**
 * Class TopicsController.
 *
 * @package namespace App\Http\Controllers\Exam;
 */
class TopicsController extends Controller
{
    /**
     * @var TopicRepository
     */
    protected $repository;

    /**
     * @var TopicValidator
     */
    protected $validator;

    /**
     * TopicsController constructor.
     *
     * @param TopicRepository $repository
     * @param TopicValidator $validator
     */
    public function __construct(TopicRepository $repository, TopicValidator $validator)
    {
        $this->repository = $repository;
        $this->validator = $validator;
    }

    /**
     * 试卷列表
     * @param Request $request
     * @return mixed
     * Author: Mead
     */
    public function index(Request $request)
    {
        $this->repository->pushCriteria(TopicCriteria::class);
        $lists = $this->repository->paginate($request->get('per_page', self::PAGE_NUM));
        return Response::success($lists);
    }

    /**
     * 添加课程
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource|void
     * @throws \Illuminate\Validation\ValidationException
     * Author: Mead
     */
    public function store(Request $request)
    {
        $this->validate($request, $this->validator->getRules(ValidatorInterface::RULE_CREATE));
        try {
            $data = $request->only(array_keys($this->validator->getRules(ValidatorInterface::RULE_CREATE)));
            $data['user_id'] = login_user_id();
            $model = $this->repository->create($data);
            return Response::success($model);
        } catch (\Exception $e) {
            return $this->errorStore($e);
        }
    }

    /**
     * 修改试题
     * @param Request $request
     * @param $id
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * @throws \Illuminate\Validation\ValidationException
     * Author: Mead
     */
    public function update(Request $request, $id)
    {
        $this->validate($request, $this->validator->getRules(ValidatorInterface::RULE_UPDATE));

        try {
            $data = $request->only(array_keys($this->validator->getRules(ValidatorInterface::RULE_UPDATE)));
            $model = $this->repository->update($data, $id);
            return Response::success($model);
        } catch (\Exception $e) {
            $this->errorStore($e);
        }
    }

    /**
     * 删除
     * @param $id
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function destroy($id)
    {
        if (!Topic::query()->where('user_id', login_user_id())->where('id', $id)->exists()) {
            return $this->error('您暂无删除权限！');
        }

        try {
            $this->repository->delete($id);
            return Response::success(null, T('successfully delete.'));
        } catch (\Exception $exception) {
            return Response::fail(T('Delete failed.'), ResponseCodeEnum::SERVICE_UPDATE_DATA_ERROR);
        }
    }

    /**
     * 试题选项
     * @param Request $request
     * Author: Mead
     */
    public function selectOptions(Request $request)
    {
        $lists = $this->repository->select(['id', 'title', 'point'])->paginate($request->get('per_page', self::PAGE_NUM));
        return Response::success($lists);
    }

    /**
     * 详情
     * @param $slug
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function show($id)
    {
        $model = $this->repository->find($id);
        if (!$model) {
            return $this->error('找不到资源');
        }
        return Response::success($model);
    }
}
