<?php

namespace App\Http\Controllers\Api\User;

use App\Contracts\Repositories\User\UserRepository;
use App\Exports\StudentExport;
use App\Exports\UserExport;
use App\Http\Controllers\Controller;
use App\Jobs\ImportStudentJob;
use App\Jobs\ImportUserJob;
use App\Repositories\Enums\ResponseCodeEnum;
use App\Repositories\Models\Base\Resource;
use App\Repositories\Models\User\User;
use App\Repositories\Validators\User\UserValidator;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Jiannei\Response\Laravel\Support\Facades\Response;
use Prettus\Validator\Contracts\ValidatorInterface;

/**
 * Class StudentsController.
 *
 * @package namespace App\Http\Controllers\Admin\User;
 */
class UserController extends Controller
{
    /**
     * @var UserRepository
     */
    protected $repository;

    /**
     * @var UserValidator
     */
    protected $validator;

    /**
     * StudentsController constructor.
     *
     * @param UserRepository $repository
     * @param UserValidator $validator
     */
    public function __construct(UserRepository $repository, UserValidator $validator)
    {
        $this->repository = $repository;
        $this->validator = $validator;
    }

    /**
     * 列表
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function index()
    {
        $lists = $this->repository->paginate(request('per_page', self::PAGE_NUM));
        return Response::success($lists);
    }

    /**
     * 添加
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource|void
     * @throws \Illuminate\Validation\ValidationException
     * Author: Mead
     */
    public function store(Request $request)
    {
        $this->validate($request, $this->validator->getRules(ValidatorInterface::RULE_CREATE));
        if (User::query()->where('username', $request->get('username'))->exists()) {
            return Response::fail('账户已存在');
        }

        try {
            $data = $request->only(array_keys($this->validator->getRules(ValidatorInterface::RULE_CREATE)));
            if (!isset($data['password']) || !$data['password']) {
                $data['password'] = $data['username'];
            }
            $data['password'] = Hash::make($data['password']);
            $model = User::query()->create($data);
            return Response::success($model);
        } catch (\Exception $e) {
            return $this->errorStore($e);
        }
    }

    /**
     * 更新个人信息
     * @param $id
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * @throws \Illuminate\Validation\ValidationException
     * Author: Mead
     */
    public function update($id, Request $request)
    {
        $this->validate($request, $this->validator->getRules(ValidatorInterface::RULE_UPDATE));
        try {
            $data = $request->only(array_keys($this->validator->getRules(ValidatorInterface::RULE_UPDATE)));
            $student = $this->repository->update($data, $id);
            return Response::success($student);
        } catch (\Exception $e) {
            $this->errorStore($e);
        }
    }

    /**
     * 详情
     * @param $id
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function show($id)
    {
        $courseCategory = $this->repository->find($id);
        return Response::success($courseCategory);
    }

    /**
     * 删除
     * @param $id
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function destroy($id)
    {
        try {
            $re = $this->repository->delete($id);
            if ($re) {
                return Response::success(null, T('successfully delete.'));
            }
            return Response::fail(T('Delete failed.'), ResponseCodeEnum::SERVICE_UPDATE_DATA_ERROR);
        } catch (\Exception $exception) {
            return Response::fail(T('Delete failed.'), ResponseCodeEnum::SERVICE_UPDATE_DATA_ERROR);
        }
    }

    /**
     * 批量删除
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function delete(Request $request)
    {
        try {
            $this->validate($request, [
                'ids' => 'required|array',
            ]);

            $ids = $request->get('ids');
            $this->repository->whereIn('id', $ids)->delete();
            return Response::success(null, T('successfully delete.'));
        } catch (\Exception $exception) {
            return Response::fail(T('Delete failed.'), ResponseCodeEnum::SERVICE_UPDATE_DATA_ERROR);
        }
    }

    /**
     * 批量导入
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * @throws \Illuminate\Validation\ValidationException
     * Author: Mead
     */
    public function import(Request $request)
    {
        $this->validate($request, [
            'id' => 'required|integer',
            'role_id' => 'required|integer'
        ]);

        $resource = Resource::query()->where('id', $request->get('id'))->first();
        if (!$resource) {
            return $this->errorFail();
        }
        $role = $request->get('role_id', 0);
        $this->dispatch(new ImportUserJob($resource['disk'], $resource['path'], $role));
        return Response::success(null, '正在导入，请稍后');
    }

    /**
     * 导出
     * Author: Mead
     */
    public function export(Request $request)
    {
        $this->validate($request, [
            'ids' => 'required|array',
            'fields' => 'sometimes|array',
        ]);
        $ids = $request->get('ids');
        $name = time();
        return (new UserExport($request->get('fields', [0, 1, 2, 3, 4, 5])))->forIds($ids)->download($name . '.xlsx', null, [
            'Access-Control-Allow-Origin' => '*',
            'Access-Control-Allow-Methods' => '*',
            'Access-Control-Allow-Headers' => 'Content-Type, Authorization, X-Requested-With',
            'Access-Control-Expose-Headers' => 'Content-Disposition',
            'Content-type' => 'application/octet-stream',
            'Content-Disposition' => 'attachment; filename=' . $name . '.xlsx',
        ]);
    }
}
